const fs = require('fs');
const fse = require("fs-extra");
const path = require('path');
const ora = require('ora');
const chalk = require('chalk');
const inquirer = require('inquirer');
const lodash = require("lodash");
const log = require('./logger');
const config = require("../../config");
const backup = require("./backup");
const utils = require("./index");
const project = require("./project");
const expand = require("./expand");
const plugin = require("../plugin/index");

const TemplateRelease = require("../../template-release");
const templateRelease = new TemplateRelease(config.cacheDirName, config.templateReleaseUrl);
const isWin = /^win/.test(process.platform);
const dirCut = isWin ? "\\" : "/";

function start() {
    let projectPath = path.resolve(process.cwd());
    let configFile = projectPath + dirCut + "eeui.config.js";
    if (!fs.existsSync(configFile)) {
        log.fatal("当前目录非eeui项目，无法进行升级操作！");
    }
    let releaseFile = projectPath + dirCut + ".eeui.release";
    let releaseConfig = utils.jsonParse(!fs.existsSync(releaseFile) ? {} : fs.readFileSync(releaseFile, 'utf8'));
    //
    let appRelease = releaseConfig.release;
    let newRelease = '';
    let spinFetch = ora('正在获取版本列表...');
    spinFetch.start();
    templateRelease.fetchReleaseVersions((err, result) => {
        spinFetch.stop();
        if (err) {
            log.fatal(err);
        }
        newRelease = result[0];
        let isLatest = false;
        if (!utils.versionFunegt(newRelease, appRelease)) {
            log.eeuis(`当前版本（${appRelease}）已是最新版本。`);
            isLatest = true;
        }
        //
        let questions = [{
            type: 'confirm',
            message: isLatest ? (`确定重新安装主框架吗？（${newRelease}）`) : (`确定开始升级主框架吗？（${appRelease} -> ${newRelease}）`),
            name: 'ok',
            default: !isLatest,
        }];
        if (!utils.isHave(releaseConfig.appName)) {
            let applicationid = "";
            questions.push({
                type: 'input',
                name: 'appName',
                default: function () {
                    return 'eeui演示';
                },
                message: "请输入App名称",
                validate: function (value) {
                    return value !== ''
                }
            }, {
                type: 'input',
                name: 'applicationID',
                default: function () {
                    return releaseConfig.applicationID || 'app.eeui.simple';
                },
                message: "请输入Android应用ID",
                validate: function (value) {
                    let pass = value.match(/^[a-zA-Z_][a-zA-Z0-9_]*[.][a-zA-Z_][a-zA-Z0-9_]*[.][a-zA-Z_][a-zA-Z0-9_]+$/);
                    if (pass) {
                        applicationid = value;
                        return true;
                    }
                    return '输入格式错误，请重新输入。';
                }
            }, {
                type: 'input',
                name: 'bundleIdentifier',
                default: function () {
                    return releaseConfig.bundleIdentifier || applicationid;
                },
                message: "请输入iOS应用ID",
                validate: function (value) {
                    let pass = value.match(/^[a-zA-Z_][a-zA-Z0-9_]*[.][a-zA-Z_][a-zA-Z0-9_]*[.][a-zA-Z_][a-zA-Z0-9_]+$/);
                    if (pass) {
                        return true;
                    }
                    return '输入格式错误，请重新输入。';
                }
            });
        }
        inquirer.prompt(questions).then(answers => {
            if (answers.ok) {
                log.eeui(`开始升级至：${newRelease}`);
                let originalData = {
                    android: {
                        versionCode: expand.androidGradle("versionCode"),
                        versionName: expand.androidGradle("versionName"),
                    },
                    ios: {
                        CFBundleVersion: expand.iosInfo("CFBundleVersion"),
                        CFBundleShortVersionString: expand.iosInfo("CFBundleShortVersionString"),
                    }
                };
                if (!utils.isHave(releaseConfig.release)) {
                    releaseConfig = lodash.merge(releaseConfig, answers);
                }
                templateRelease.fetchRelease(newRelease, function (error, releasePath) {
                    if (error) {
                        log.fatal(error);
                    }
                    let releasePackageFile = path.resolve(releasePath, "package.json");
                    if (fs.existsSync(releasePackageFile)) {
                        let releasePackageInfo = utils.jsonParse(fs.readFileSync(releasePackageFile, 'utf8'));
                        let current = require('../../package.json').version;
                        let eeuiclimin = releasePackageInfo.eeuiclimin;
                        if (utils.isHave(eeuiclimin) && utils.versionFunegt(eeuiclimin, current)) {
                            log.error(`当前${chalk.underline(`eeui-cli@${current}`)}版本过低，无法进行升级，请升级至${chalk.underline(`eeui-cli@${eeuiclimin}`)}或以上！`);
                        }
                    }
                    log.eeui(`备份项目开发文件...`);
                    backup.backup((ret, backPath) => {
                        if (!ret) {
                            log.fatal(`备份失败：${backPath}`);
                        }
                        log.eeui(`备份成功`);
                        log.eeui(`升级新版本文件...`);
                        backup.updateMainFrame(releasePath, projectPath);
                        log.eeui(`升级成功`);
                        log.eeui(`恢复项目开发文件...`);
                        backup.recoveryHandler(backPath, (rec, msg) => {
                            if (!rec) {
                                log.fatal(`恢复失败：${msg}`);
                            }
                            log.eeui(`恢复成功`);
                            fse.removeSync(backPath);
                            //
                            releaseConfig.release = newRelease;
                            project.initConfig(projectPath, releaseConfig);
                            expand.androidGradle("versionCode", originalData.android.versionCode);
                            expand.androidGradle("versionName", originalData.android.versionName);
                            expand.iosInfo("CFBundleVersion", originalData.ios.CFBundleVersion);
                            expand.iosInfo("CFBundleShortVersionString", originalData.ios.CFBundleShortVersionString);
                            //
                            plugin.repair(() => {
                                let packageFile = path.resolve(process.cwd(), "package.json");
                                if (fs.existsSync(packageFile)) {
                                    let packageInfo = fs.readFileSync(packageFile, 'utf8');
                                    let releasePackageFile = path.resolve(releasePath, "package.json");
                                    if (fs.existsSync(releasePackageFile)) {
                                        let releasePackageInfo = utils.jsonParse(fs.readFileSync(releasePackageFile, 'utf8'));
                                        let projectPackageInfo = utils.jsonParse(packageInfo);
                                        fs.writeFileSync(packageFile, JSON.stringify(lodash.merge(projectPackageInfo, releasePackageInfo), null, "\t"), {encode: 'utf-8'})
                                    }else{
                                        packageInfo = packageInfo.replace(/"version"\s*:\s*"(.*?)"/g, `"version": "${newRelease}"`);
                                        fs.writeFileSync(packageFile, packageInfo, {encode: 'utf-8'})
                                    }
                                }
                                //
                                log.eeuis(`主框架升级至最新版本（${newRelease}）成功。`);
                                log.sep();
                                log.eeui("您可以运行一下命令开始。");
                                log.eeui(chalk.white(`1. npm install`));
                                log.eeui(chalk.white(`2. npm run dev`));
                            });
                        }, ['src']);
                    }, ['src']);
                });
            } else {
                log.fatal(isLatest ? `放弃操作！` : `放弃升级！`);
            }
        }).catch(console.error);
    });
}

module.exports = {start};
